uses
  SDL3;
label
  Cleanup,
  FailedWindow,
  FailedRenderer;
var
  Window:   PSDL_Window;
  Renderer: PSDL_Renderer;
  Event:    TSDL_Event;
begin
  Window := SDL_CreateWindow('Window test', 640, 480, SDL_WINDOW_RESIZABLE);

  if Window = nil then
  begin
    WriteLn('SDL_CreateWindow failed: "', SDL_GetError(), '".');
    goto FailedWindow;
  end;

  Renderer := SDL_CreateRenderer(Window, 'opengl');

  if Renderer = nil then
  begin
    WriteLn('SDL_CreateRenderer failed: "', SDL_GetError(), '".');
    goto FailedRenderer;
  end;

  while True do
  begin
    while SDL_PollEvent(@Event) do
    case Event._Type of
      SDL_EVENT_QUIT: goto Cleanup;

      SDL_EVENT_KEY_DOWN:
      case Event.Key.Scancode of
        SDL_SCANCODE_F: SDL_SetWindowFullscreen(Window, True);
        SDL_SCANCODE_W: SDL_SetWindowFullscreen(Window, False);
      end;

      SDL_EVENT_WINDOW_MINIMIZED:        WriteLn(SDL_GetTicks():7, '  SDL_EVENT_WINDOW_MINIMIZED');
      SDL_EVENT_WINDOW_MAXIMIZED:        WriteLn(SDL_GetTicks():7, '  SDL_EVENT_WINDOW_MAXIMIZED');
      SDL_EVENT_WINDOW_RESTORED:         WriteLn(SDL_GetTicks():7, '  SDL_EVENT_WINDOW_RESTORED');
      SDL_EVENT_WINDOW_ENTER_FULLSCREEN: WriteLn(SDL_GetTicks():7, '  SDL_EVENT_WINDOW_ENTER_FULLSCREEN');
      SDL_EVENT_WINDOW_LEAVE_FULLSCREEN: WriteLn(SDL_GetTicks():7, '  SDL_EVENT_WINDOW_LEAVE_FULLSCREEN');

      SDL_EVENT_WINDOW_MOVED:
        WriteLn(SDL_GetTicks():7, '  SDL_EVENT_WINDOW_MOVED: ', Event.Window.Data1, ',', Event.Window.Data2);

      SDL_EVENT_WINDOW_RESIZED:
        WriteLn(SDL_GetTicks():7, '  SDL_EVENT_WINDOW_RESIZED: ', Event.Window.Data1, ',', Event.Window.Data2);

      SDL_EVENT_WINDOW_PIXEL_SIZE_CHANGED:
        WriteLn(SDL_GetTicks():7, '  SDL_EVENT_WINDOW_PIXEL_SIZE_CHANGED: ', Event.Window.Data1, ',', Event.Window.Data2);
    end;

    SDL_SetRenderDrawColor (Renderer, 225, 225, 225, 255);
    SDL_RenderClear        (Renderer);
    SDL_RenderPresent      (Renderer);
    SDL_Delay(20);
  end;

Cleanup:
  SDL_DestroyRenderer(Renderer);

FailedRenderer:
  SDL_DestroyWindow(Window);

FailedWindow:
  WriteLn('Press ENTER to finish...');
  ReadLn();
end.

